@@ -22,6 +22,7 @@ from construct import (
     Default,
     FixedSized,
     Flag,
+    FocusedSeq,
     FormatField,
     GreedyBytes,
     GreedyRange,
@@ -145,6 +146,16 @@ def convert(
             message_name=message_name,
         )
 
+    elif isinstance(con, FocusedSeq):
+        subcon = getattr(con, con.parsebuildfrom)
+        convert(
+            subcon.subcon,
+            visitor,
+            field_name=field_name,
+            struct_name=struct_name,
+            many=False,
+        )
+
     elif con is Pass:
         return
 
