@@ -439,7 +439,9 @@ def camelcase(field_name):
 
 
 def snakecase(camel_input):
-    words = re.findall(r"[A-Z]?[a-z]+|[A-Z]{2,}(?=[A-Z][a-z]|\d|\W|$)|\d+", camel_input)
+    words = re.findall(
+        r"[A-Z]?[a-z\d]+|[A-Z]{2,}(?=[A-Z][a-z]|\d|\W|$)|\d+", camel_input
+    )
     return "_".join(map(str.lower, words))
 
 
